import React, { useEffect, useState } from 'react';
import SearchResults from './search-results';
import Loader from './loader';
import messageService from '../services/MessageService';
import { CONFIGS } from '../client-config'

import { Button, Input, Col, Row } from 'antd';
import { SearchOutlined } from '@ant-design/icons';

const Search = () => {

    const INITIAL_STATE = { searchQuery: '', facets: [], filters: null, pageSize: 10, pageOffset: 0 };

    const [payload, setPayload] = useState(INITIAL_STATE);
    const [searchResults, setSearchResults] = useState([]);
    const [facets, setfacets] = useState([]);
    const [isLoader, setIsLoader] = useState(false);
    const [totalRecords, setTotalRecords] = useState(0);

    const handleOnChange = (e) => {
        setPayload(prevState => ({
            ...prevState,
            searchQuery: e.target.value,
            filters: null
        }));
    };

    const handleOnSearch = async () => {
        const { searchQuery, facets, filters, pageSize, pageOffset } = payload;
        if (searchQuery) {
            setIsLoader(true);
            const apiEndpoint = 'https://api-dot-abs-poc-np-prj-01-3f2a.uc.r.appspot.com';
            // const apiEndpoint = 'http://localhost:8080';
            const url =  `${apiEndpoint}/search?text=${searchQuery}&pageSize=${pageSize}&pageOffset=${pageOffset}${facets.length? `&facets=${JSON.stringify(facets)}` : ''}${filters && Object.keys(filters).length? `&filters=${JSON.stringify(filters)}` : ''}`;  
            const searchResults = await fetch(url)
                .then(res => res.json())
                .then(_res => {
                    if (Array.isArray(_res) && _res.length === 0 ) {
                        messageService.info('', 'No Data Found');
                    }
                    return _res;
                })
                .catch(err => console.log(err))
                .finally(() => setIsLoader(false));
            setSearchResults(searchResults.results?.length ? [...searchResults.results] : []);
            setTotalRecords(searchResults.totalSize? searchResults.totalSize : 0);
            !Object.keys(filters || {}).length && setfacets(searchResults.facets?.length ? [...searchResults.facets] : []); // for not updating facets while filtering
        }
    };

    const handleImageClick = () => {
        setPayload(INITIAL_STATE);
        setSearchResults([]);
        setfacets([]);
    };

    const onSelectFacet = (val) => {
        setPayload(prevState => ({
            ...prevState,
            facets: val,
        }));
    }

    const onSelectFilter = (val) => {
        setPayload(prevState => ({
            ...prevState,
            filters: val,
        }));
    }

    const onChangePageSize = (val) => {
       const { current, pageSize } = val;
       setPayload(prevState => ({
            ...prevState,
            pageSize: pageSize,
            pageOffset: pageSize * (current - 1),
        }));
      };
    
    const onChangePage = (val) => {
        const { current, pageSize } = val;
        setPayload(prevState => ({
            ...prevState,
            pageOffset: pageSize * (current - 1),
        }));
    };

    useEffect(() => {
        if (payload.filters != null) {
            handleOnSearch();
        }
    }, [payload.filters]);

    useEffect(() => {
        payload.facets.length ? handleOnSearch() : setfacets([]);
    }, [payload.facets]);

    useEffect(() => {
        handleOnSearch();
    }, [payload.pageSize, payload.pageOffset]);

    return (
        <>
            {searchResults.length > 0 &&
                <>
                    <div>
                        <Row style={{
                            alignItems: 'center'
                        }}
                        >
                            <Col span={5} style={{
                                margin: '1rem'
                            }}
                            >
                                <span style={{
                                    margin: '1rem 0',
                                    height: 'auto',
                                }}
                                    onClick={() => handleImageClick()}
                                >
                                    <img src={CONFIGS.client_logo} alt={CONFIGS.title} style={{
                                    }}
                                    />
                                </span>
                            </Col>
                            <Col span={12} style={{
                                margin: '0 1rem'
                            }}
                            >
                                <Input
                                    size='large'
                                    placeholder="Input Search Text"
                                    value={payload.searchQuery}
                                    allowClear
                                    prefix={<SearchOutlined style={{ color: "#d9d9d9", margin: '0 0.5rem' }} />}
                                    onChange={handleOnChange}
                                    onPressEnter={() => handleOnSearch()}
                                />
                            </Col>
                            <Col span={2}>   {isLoader && <Loader />}  </Col>
                        </Row>
                    </div>
                    <SearchResults searchResults={searchResults} facets={facets} totalRecords={totalRecords} pageSize={payload.pageSize}
                        onSelectFacet={(val) => onSelectFacet(val)}
                        onSelectFilter={(val) => onSelectFilter(val)}
                        onChangePageSize={(val) => onChangePageSize(val)}
                        onChangePage={(val) => onChangePage(val)}
                    />
                </>
            }
            {searchResults.length === 0 &&
                <div style={{
                    textAlign: 'center',
                    marginTop: '1rem',
                    top: '50%'
                }}>
                    <Row>
                        <Col span={12} offset={6}>
                            <div className='App'>
                                <span>
                                    <img src={CONFIGS.client_logo} alt={CONFIGS.title} style={{
                                        width: '12rem',
                                        height: 'auto',
                                        marginBottom: '2rem'
                                    }}
                                    />
                                </span>

                                <Input
                                    size='large'
                                    placeholder="Input Search Text"
                                    value={payload.searchQuery}
                                    allowClear
                                    prefix={<SearchOutlined style={{ color: "#d9d9d9", margin: '0 0.5rem' }} />}
                                    onChange={handleOnChange}
                                    onPressEnter={() => handleOnSearch()}
                                />
                                <Button type='primary' style={{
                                    margin: '2rem 1rem',
                                    minWidth: '7rem',
                                    minHeight: '2.5rem',
                                    background: '#b5904a'
                                }}
                                    onClick={() => handleOnSearch()}
                                > Search </Button>
                                { isLoader && <Loader />}
                            </div>
                        </Col>
                    </Row>
                </div>
            }
        </>
    )
}

export default Search;